# imports
# =============================================================================
# standard
# *****************************************************************************
import types
import os

# pygame
# *****************************************************************************
import pygame

# ablauf
# *****************************************************************************
import ablauf


class Kernel(types.ModuleType):
    __screen = None
    __clock = None
    __sync = False


    class __metaclass__(type):
        @property
        def screen(cls):
            return cls.__screen

        @screen.setter
        def screen(cls, value):
            cls.__screen = value

        @property
        def clock(cls):
            return cls.__clock

        @clock.setter
        def clock(cls, value):
            cls.__clock = value

        @property
        def sync(cls):
            return cls.__sync

        @sync.setter
        def sync(cls, value):
            cls.__sync = value


def init():
    # Pygame initialization
    pygame.init()
    pygame.joystick.init()

    _flags = 0

    for flag in ablauf.Data.configuration["flags"]:
        exec("_flags += pygame." + flag)

    # create a screen
    Kernel.screen = pygame.display.set_mode((ablauf.Data.configuration["width"], ablauf.Data.configuration["height"]),_flags)

    # set windows name
    pygame.display.set_caption(ablauf.Data.configuration["name"] + " " + ablauf.Data.configuration["version"] + " - " + ablauf.Data.configuration["author"] + " " + ablauf.Data.configuration["year_of_release"])

    # show or hide cursor
    pygame.mouse.set_visible(ablauf.Data.configuration["cursor_visible"])

    # set clock
    Kernel.clock = pygame.time.Clock()

    # init sync
    Kernel.sync = False

    # pygame constants
    Kernel.event_quit = pygame.QUIT
    Kernel.event_key_pressed = pygame.KEYDOWN
    Kernel.event_escape_pressed = pygame.K_ESCAPE


def reinitialize_controllers():
    pygame.joystick.quit()
    pygame.joystick.init()


def screen_flip():
    pygame.display.flip()


def clock_sync(fps):
    Kernel.clock.tick(fps)


def get_events():
    _events = pygame.event.get()

    if _events.__len__() == 0:
        some_event = pygame.event.Event(pygame.USEREVENT + 2)
        _events = [some_event]

    return _events


def get_key_object(key):
    _key = None
    exec ("_key = pygame." + key)

    return _key


def test_if_aborted(event):
    # test if the program was aborted
    if event.type == pygame.QUIT:
        # windows closed
        ablauf.quit()

    elif event.type == pygame.KEYDOWN:
        # escape pressed
        if event.key == pygame.K_ESCAPE:
            ablauf.quit()

# **************************************************************
# Helper functions
# **************************************************************
# --------------------------------------------------------------
#  Draw a rectangle helper function
# --------------------------------------------------------------
def filled_rectangle(position_x, position_y, width, height, color, corner_deep=0, corner_color=(0, 0, 0)):
    _background = pygame.Surface((width, height))
    _background = _background.convert()
    _background.fill(color)

    _corner1 = None
    _corner2 = None
    _corner3 = None

    if corner_deep > 0:
        _corner1 = pygame.Surface((1, 1))
        _corner1 = _corner1.convert()
        _corner1.fill(corner_color)
        _corner2 = pygame.Surface((corner_deep, 1))
        _corner2 = _corner2.convert()
        _corner2.fill(corner_color)
        _corner3 = pygame.Surface((1, corner_deep))
        _corner3 = _corner3.convert()
        _corner3.fill(corner_color)

    Kernel.screen.blit(_background, (position_x, position_y))

    if corner_deep > 0:
        # top left corner
        Kernel.screen.blit(_corner1, (position_x + 1, position_y + 1))
        Kernel.screen.blit(_corner2, (position_x, position_y))
        Kernel.screen.blit(_corner3, (position_x, position_y))
        # top right corner
        Kernel.screen.blit(_corner1, (position_x + width - 2, position_y + 1))
        Kernel.screen.blit(_corner2, (position_x + width - corner_deep, position_y))
        Kernel.screen.blit(_corner3, (position_x + width - 1, position_y))
        # down left corner
        Kernel.screen.blit(_corner1, (position_x + 1, position_y + height - 2))
        Kernel.screen.blit(_corner2, (position_x, position_y + height - 1))
        Kernel.screen.blit(_corner3, (position_x, position_y + height - corner_deep))
        # down right corner
        Kernel.screen.blit(_corner1, (position_x + width - 2, position_y + height - 2))
        Kernel.screen.blit(_corner2, (position_x + width - corner_deep, position_y + height - 1))
        Kernel.screen.blit(_corner3, (position_x + width - 1, position_y + height - corner_deep))


def scalable_text(message, position_x, position_y, font=None, font_size=36, color=(0, 0, 0), background_width=None, background_height=None):
    _font = pygame.font.Font(font, font_size)
    _text = _font.render(message, 1, color)

    if background_width is None:
        _background_width = pygame.display.get_surface().get_rect().width
    else:
        _background_width = background_width

    if background_height is None:
        _background_height = pygame.display.get_surface().get_rect().height
    else:
        _background_height = background_height

    if position_x is None and position_y is None:
        _text_pos = _text.get_rect(centerx=_background_width / 2, centery=_background_height / 2)
    elif position_x is None:
        _text_pos = _text.get_rect(centerx=_background_width / 2)
        _text_pos[1] = position_y
    elif position_y is None:
        _text_pos = _text.get_rect(centery=_background_height / 2)
        _text_pos[0] = position_x
    elif position_x is not None and background_width is not None:
        _text_pos = (position_x + _text.get_rect(centerx=_background_width / 2)[0] , position_y)
    else:
        _text_pos = (position_x, position_y)

    Kernel.screen.blit(_text, _text_pos)


def load_sound(name):
    class NoneSound:
        def play(self): pass

    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()
    fullname = os.path.join('media/sfx', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except Exception as ex:
        print('Cannot load sound:{0} because:'.format(fullname, ex))
        raise SystemExit
    return sound


def get_image(path):
    if not path in ablauf.Automate.image_cache:
        if os.path.exists(path):
            _surface = pygame.image.load(path)
            ablauf.Automate.image_cache[path] = _surface
            print("loaded {0} into the cache".format(path))
        else:
            #print("[Warning] File not found:" + path)
            return None

    return ablauf.Automate.image_cache[path]